/*
 * Name:    mp_digital_input.c
 *
 * Purpose: Python module interface to generic MX digital input functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_digital_input.h"
#include "mx_handle.h"
#include "mp.h"

MX_EXPORT PyObject *
MpPrivate_digital_input_read_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	unsigned long value;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_digital_input_read( record, &value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) value );

	return result;
}

