/*
 * Name:    mp_vme.c
 *
 * Purpose: Python module interface to generic MX port I/O functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_types.h"
#include "mx_vme.h"
#include "mx_handle.h"
#include "mp.h"

MX_EXPORT PyObject *
MpPrivate_vme_in8_cmd(PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long crate, address_mode, address;
	mx_uint8_type value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llll",
				&handle, &crate, &address_mode, &address );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_in8( record, crate, address_mode, address, &value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) value );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_vme_in16_cmd(PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long crate, address_mode, address;
	mx_uint16_type value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llll",
				&handle, &crate, &address_mode, &address );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_in16( record, crate, address_mode, address, &value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", (int) value );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_vme_in32_cmd(PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long crate, address_mode, address;
	mx_uint32_type value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llll",
				&handle, &crate, &address_mode, &address );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_in32( record, crate, address_mode, address, &value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) value );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_vme_out8_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long crate, address_mode, address;
	int value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lllli",
			&handle, &crate, &address_mode, &address, &value );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_out8( record,
				crate, address_mode, address,
				(mx_uint8_type) value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_vme_out16_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long crate, address_mode, address;
	int value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lllli",
			&handle, &crate, &address_mode, &address, &value );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_out16( record,
				crate, address_mode, address,
				(mx_uint16_type) value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_vme_out32_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long crate, address_mode, address, value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lllll",
			&handle, &crate, &address_mode, &address, &value );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_out32( record,
				crate, address_mode, address,
				(mx_uint32_type) value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_vme_multi_in8_cmd(PyObject *self, PyObject *args )
{
	const char cname[] = "multi_in8";

	PyObject *values_tuple, *value_object;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long i, crate, address_mode, address, num_values;
	mx_uint8_type *value_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lllll",
			&handle, &crate, &address_mode, &address, &num_values );

	if ( parse_status == 0 )
		return NULL;

	value_array = (mx_uint8_type *)
			malloc( num_values * sizeof(mx_uint8_type) );

	if ( value_array == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
	"%s: Ran out of memory trying to allocate a %ld element array.",
			cname, num_values );
		return NULL;
	}

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS ) {
		mx_free( value_array );
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );
	}

	/* Get the values from the VME crate. */

	mx_status = mx_vme_multi_in8( record,
					crate, address_mode, address,
					num_values, value_array );

	if ( mx_status.code != MXE_SUCCESS ) {
		mx_free( value_array );
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );
	}

	/* Build a tuple to contain the returned values. */

	values_tuple = PyTuple_New( (int) num_values );

	if ( values_tuple == NULL ) {
		mx_free( value_array );
		return NULL;
	}

	for ( i = 0; i < num_values; i++ ) {

		value_object = PyInt_FromLong( (long) value_array[i] );

		if ( value_object == NULL ) {
			Py_DECREF( values_tuple );
			mx_free( value_array );
			return NULL;
		}

		PyTuple_SET_ITEM( values_tuple, i, value_object );
	}

	mx_free(value_array);

	return values_tuple;
}

MX_EXPORT PyObject *
MpPrivate_vme_multi_in16_cmd(PyObject *self, PyObject *args )
{
	const char cname[] = "multi_in16";

	PyObject *values_tuple, *value_object;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long i, crate, address_mode, address, num_values;
	mx_uint16_type *value_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lllll",
			&handle, &crate, &address_mode, &address, &num_values );

	if ( parse_status == 0 )
		return NULL;

	value_array = (mx_uint16_type *)
			malloc( num_values * sizeof(mx_uint16_type) );

	if ( value_array == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
	"%s: Ran out of memory trying to allocate a %ld element array.",
			cname, num_values );
		return NULL;
	}

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS ) {
		mx_free( value_array );
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );
	}

	/* Get the values from the VME crate. */

	mx_status = mx_vme_multi_in16( record,
					crate, address_mode, address,
					num_values, value_array );

	if ( mx_status.code != MXE_SUCCESS ) {
		mx_free( value_array );
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );
	}

	/* Build a tuple to contain the returned values. */

	values_tuple = PyTuple_New( (int) num_values );

	if ( values_tuple == NULL ) {
		mx_free( value_array );
		return NULL;
	}

	for ( i = 0; i < num_values; i++ ) {

		value_object = PyInt_FromLong( (long) value_array[i] );

		if ( value_object == NULL ) {
			Py_DECREF( values_tuple );
			mx_free( value_array );
			return NULL;
		}

		PyTuple_SET_ITEM( values_tuple, i, value_object );
	}

	mx_free(value_array);

	return values_tuple;
}

MX_EXPORT PyObject *
MpPrivate_vme_multi_in32_cmd(PyObject *self, PyObject *args )
{
	const char cname[] = "multi_in32";

	PyObject *values_tuple, *value_object;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long i, crate, address_mode, address, num_values;
	mx_uint32_type *value_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lllll",
			&handle, &crate, &address_mode, &address, &num_values );

	if ( parse_status == 0 )
		return NULL;

	value_array = (mx_uint32_type *)
			malloc( num_values * sizeof(mx_uint32_type) );

	if ( value_array == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
	"%s: Ran out of memory trying to allocate a %ld element array.",
			cname, num_values );
		return NULL;
	}

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS ) {
		mx_free( value_array );
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );
	}

	/* Get the values from the VME crate. */

	mx_status = mx_vme_multi_in32( record,
					crate, address_mode, address,
					num_values, value_array );

	if ( mx_status.code != MXE_SUCCESS ) {
		mx_free( value_array );
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );
	}

	/* Build a tuple to contain the returned values. */

	values_tuple = PyTuple_New( (int) num_values );

	if ( values_tuple == NULL ) {
		mx_free( value_array );
		return NULL;
	}

	for ( i = 0; i < num_values; i++ ) {

		value_object = PyInt_FromLong( (long) value_array[i] );

		if ( value_object == NULL ) {
			Py_DECREF( values_tuple );
			mx_free( value_array );
			return NULL;
		}

		PyTuple_SET_ITEM( values_tuple, i, value_object );
	}

	mx_free(value_array);

	return values_tuple;
}

MX_EXPORT PyObject *
MpPrivate_vme_multi_out8_cmd(PyObject *self, PyObject *args )
{
	const char cname[] = "multi_out8";

	PyObject *values_tuple, *value_object;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long i, crate, address_mode, address, num_values;
	mx_uint8_type *value_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llllO",
		&handle, &crate, &address_mode, &address, &values_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* The last object passed to us should be a tuple. */

	if ( PyTuple_Check( values_tuple ) == 0 )
		return NULL;

	/* Find out how many values are in the tuple. */

	num_values = (long) PyTuple_Size( values_tuple );

	if ( num_values <= 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
		"%s: The values tuple passed has the illegal size of %ld.",
			cname, num_values );
		return NULL;
	}

	/* Allocate an array to hold the values we are going to send
	 * to the VME crate.
	 */

	value_array = (mx_uint8_type *)
			malloc( num_values * sizeof(mx_uint8_type) );

	if ( value_array == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
	"%s: Ran out of memory trying to allocate a %ld element array.",
			cname, num_values );
		return NULL;
	}

	/* Copy the values from the tuple to the array. */

	for ( i = 0; i < num_values; i++ ) {

		value_object = PyTuple_GetItem( values_tuple, i );

		if ( value_object == NULL ) {
			mx_free( value_array );
			return NULL;
		}

		if ( PyInt_Check( value_object ) == 0 ) {
			mx_free( value_array );
			return NULL;
		}

		value_array[i] = (mx_uint8_type) PyInt_AsLong( value_object );
	}

	/* Now we can finally send the bytes out to the hardware. */

	mx_status = mx_vme_multi_out8( record,
				crate, address_mode, address,
				(unsigned long) num_values, value_array );

	mx_free( value_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_vme_multi_out16_cmd(PyObject *self, PyObject *args )
{
	const char cname[] = "multi_out16";

	PyObject *values_tuple, *value_object;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long i, crate, address_mode, address, num_values;
	mx_uint16_type *value_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llllO",
		&handle, &crate, &address_mode, &address, &values_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* The last object passed to us should be a tuple. */

	if ( PyTuple_Check( values_tuple ) == 0 )
		return NULL;

	/* Find out how many values are in the tuple. */

	num_values = (long) PyTuple_Size( values_tuple );

	if ( num_values <= 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
		"%s: The values tuple passed has the illegal size of %ld.",
			cname, num_values );
		return NULL;
	}

	/* Allocate an array to hold the values we are going to send
	 * to the VME crate.
	 */

	value_array = (mx_uint16_type *)
			malloc( num_values * sizeof(mx_uint16_type) );

	if ( value_array == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
	"%s: Ran out of memory trying to allocate a %ld element array.",
			cname, num_values );
		return NULL;
	}

	/* Copy the values from the tuple to the array. */

	for ( i = 0; i < num_values; i++ ) {

		value_object = PyTuple_GetItem( values_tuple, i );

		if ( value_object == NULL ) {
			mx_free( value_array );
			return NULL;
		}

		if ( PyInt_Check( value_object ) == 0 ) {
			mx_free( value_array );
			return NULL;
		}

		value_array[i] = (mx_uint16_type) PyInt_AsLong( value_object );
	}

	/* Now we can finally send the bytes out to the hardware. */

	mx_status = mx_vme_multi_out16( record,
				crate, address_mode, address,
				(unsigned long) num_values, value_array );

	mx_free( value_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_vme_multi_out32_cmd(PyObject *self, PyObject *args )
{
	const char cname[] = "multi_out32";

	PyObject *values_tuple, *value_object;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long i, crate, address_mode, address, num_values;
	mx_uint32_type *value_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llllO",
		&handle, &crate, &address_mode, &address, &values_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* The last object passed to us should be a tuple. */

	if ( PyTuple_Check( values_tuple ) == 0 )
		return NULL;

	/* Find out how many values are in the tuple. */

	num_values = (long) PyTuple_Size( values_tuple );

	if ( num_values <= 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
		"%s: The values tuple passed has the illegal size of %ld.",
			cname, num_values );
		return NULL;
	}

	/* Allocate an array to hold the values we are going to send
	 * to the VME crate.
	 */

	value_array = (mx_uint32_type *)
			malloc( num_values * sizeof(mx_uint32_type) );

	if ( value_array == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
	"%s: Ran out of memory trying to allocate a %ld element array.",
			cname, num_values );
		return NULL;
	}

	/* Copy the values from the tuple to the array. */

	for ( i = 0; i < num_values; i++ ) {

		value_object = PyTuple_GetItem( values_tuple, i );

		if ( value_object == NULL ) {
			mx_free( value_array );
			return NULL;
		}

		if ( PyInt_Check( value_object ) == 0 ) {
			mx_free( value_array );
			return NULL;
		}

		value_array[i] = (mx_uint32_type) PyInt_AsLong( value_object );
	}

	/* Now we can finally send the bytes out to the hardware. */

	mx_status = mx_vme_multi_out32( record,
				crate, address_mode, address,
				(unsigned long) num_values, value_array );

	mx_free( value_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_vme_get_read_address_increment_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle, crate;
	unsigned long address_increment;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &crate );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_get_read_address_increment( record,
						crate, &address_increment );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) address_increment );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_vme_set_read_address_increment_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	int parse_status;
	long handle, crate, address_increment;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lll",
				&handle, &crate, &address_increment );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_set_read_address_increment( record, crate,
					(unsigned long) address_increment );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_vme_get_write_address_increment_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle, crate;
	unsigned long address_increment;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &crate );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_get_write_address_increment( record,
						crate, &address_increment );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) address_increment );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_vme_set_write_address_increment_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	int parse_status;
	long handle, crate, address_increment;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lll",
				&handle, &crate, &address_increment );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_vme_set_write_address_increment( record, crate,
					(unsigned long) address_increment );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

